Attribute VB_Name = "DataAccess"
Option Explicit

Public Function MainPage() As Visio.Page
  Set MainPage = Application.ActiveDocument.Pages(1)
End Function

Public Function GetRankMaster(nRank&) As Visio.Master
  Dim iMaster As Visio.Master: Set iMaster = FindMaster(ThisDocument, MASTER_RANK & nRank)
  If iMaster Is Nothing Then
    Set GetRankMaster = GetRankMaster(nRank - 1)
  Else
    Set GetRankMaster = iMaster
  End If
End Function

Public Function GetConnectionMaster(nMode As TLayout) As Visio.Master
  Select Case nMode
    Case T_LAYOUT_HORIZONTAL: Set GetConnectionMaster = FindMaster(ThisDocument, MASTER_CONNECT_HOR)
    Case T_LAYOUT_VERTICAL: Set GetConnectionMaster = FindMaster(ThisDocument, MASTER_CONNECT_VERT)
    Case T_LAYOUT_COLUMN_LEFT: Set GetConnectionMaster = FindMaster(ThisDocument, MASTER_CONNECT_HOR)
    Case T_LAYOUT_COLUMN_RIGHT: Set GetConnectionMaster = FindMaster(ThisDocument, MASTER_CONNECT_HOR)
    Case Else: Set GetConnectionMaster = FindMaster(ThisDocument, MASTER_CONNECT_VERT)
  End Select
End Function

Public Function GetRankWidth(nRank&) As Double
  GetRankWidth = GetRankMaster(nRank).Shapes(1).CellsU("Width")
End Function

Public Function GetRanksBaseY() As Scripting.Dictionary
  Dim iBases As New Scripting.Dictionary
  Dim nRank&
  Dim baseY As Double: baseY = 0
  For nRank = 1 To TREE_MAX_RANK Step 1
    Dim iPrototype As Visio.Shape: Set iPrototype = GetRankMaster(nRank).Shapes(1)
    If Not iPrototype Is Nothing Then
      iBases(nRank) = baseY - iPrototype.CellsU("Height") / 2#
      baseY = baseY - iPrototype.CellsU("Height") - GAP_RANK
    End If
  Next nRank
  Set GetRanksBaseY = iBases
End Function

Public Function GetChildrenOf(target As Visio.Shape) As Collection
  Set GetChildrenOf = New Collection
  Dim cnt As Visio.Connect
  For Each cnt In target.FromConnects
    Dim connector As Visio.Shape: Set connector = cnt.FromSheet
    If connector.Connects(1).ToSheet Is target And connector.Connects.Count = 2 Then _
      Call GetChildrenOf.Add(connector.Connects(2).ToSheet)
  Next cnt
End Function

Public Function ExrtactXLPayload(iSource As Excel.Worksheet, nRow&) As VisioNode
  Dim iPayload As New VisioNode
  iPayload.text_ = iSource.Cells(nRow, 2)
  If iSource.Cells(nRow, 1).Interior.ColorIndex <> -4142 Then _
    iPayload.color_ = iSource.Cells(nRow, 1).Interior.Color
  Set ExrtactXLPayload = iPayload
End Function

Public Function ExtractWordPayload(iSource As Word.Range) As VisioNode
  Dim sText$: sText = TrimWhitespace(iSource.Text)
  If sText = vbNullString Then _
    Exit Function
    
  Dim nColor&: nColor = iSource.Shading.BackgroundPatternColor
  nColor = IIf(nColor = wdColorAutomatic, COLOR_DEFAULT, ColorGetRGB(iSource.Font.Shading.BackgroundPatternColor, iSource.Document))
  
  Dim iPayload As New VisioNode
  iPayload.text_ = sText
  iPayload.color_ = nColor
  Set ExtractWordPayload = iPayload
End Function

Public Function ExtractVsoPayload(target As Visio.Shape) As VisioNode
  Dim iPayload As New VisioNode
  iPayload.text_ = target.Text
  iPayload.color_ = ConvertStringToRGB(target.CellsU("FillForegnd").ResultStrU(""))
  Set iPayload.shape_ = target
  Set ExtractVsoPayload = iPayload
End Function

Public Function WordStyleForRank(wordDoc As Word.Document, nRank As Integer) As String
  WordStyleForRank = ""
  If nRank < 1 Or nRank > WORD_HEADING_MAX_LEVEL Then _
    Exit Function
  
  Dim sStyle$: sStyle = " " & nRank & " "
  If WordStyleExists(wordDoc, sStyle) Then
    WordStyleForRank = sStyle
  Else
    WordStyleForRank = " " & nRank
  End If
End Function

