VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "LayoutConstruction"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private page_ As Visio.Page
Private data_ As CDS_StaticHierarchy
Private autoVertical_ As Boolean

Private width_ As Scripting.Dictionary

Private Const SIZING_FACTOR_LIMIT = 1.3

Public Function Init(iPage As Visio.Page, ByRef iData As CDS_StaticHierarchy, bAutoVertical As Boolean)
  Set page_ = iPage
  Set data_ = iData
  autoVertical_ = bAutoVertical
End Function

Public Function ConstructHNodes()
  Dim ranksLast As New Scripting.Dictionary
  Dim ranksY As Scripting.Dictionary: Set ranksY = GetRanksBaseY
  Dim lastInsert As Visio.Shape
  
  Dim iNode As CDS_NodeSH
  Dim iParent As Visio.Shape
  For Each iNode In data_.nodes_
    Dim nRank&: nRank = iNode.rank_
    Dim nLayout As TLayout: nLayout = GetLayoutFor(iNode.parent_)
    Dim iNewShape As Visio.Shape: Set iNewShape = CreateNodeShape(iNode)
    Set iNode.data_.shape_ = iNewShape
    
    If iNode.parent_ Is Nothing Then
      Set iParent = Nothing
    Else
      Set iParent = iNode.parent_.data_.shape_
    End If
    
    If nLayout = T_LAYOUT_HORIZONTAL Then
      If Not ranksLast.Exists(nRank) Then
        iNewShape.CellsU("PinY") = ranksY(nRank) - iNewShape.CellsU("Height") / 2#
      Else
        Call MoveShapeAfterHorizontal(iNewShape, ranksLast(nRank))
      End If
      Set ranksLast(nRank) = iNewShape
      Set lastInsert = Nothing
    Else
      If lastInsert Is Nothing Then
        Call MoveShapeAfterVertical(iNewShape, iParent, GAP_RANK)
      Else
        Call MoveShapeAfterVertical(iNewShape, lastInsert, GAP_VERTICAL)
      End If
      Set lastInsert = iNewShape
    End If
    
    If iNode.children_.Count = 0 Then _
      Call TryOptimizeHeight(iNewShape, iNode.rank_, nLayout)

  If Not iParent Is Nothing Then _
    Call ConnectNodes(iParent, iNewShape, nLayout)
    
    Call CSE_ProgressBar.IncrementA
  Next iNode
End Function

Public Function CalculateWidth()
  Set width_ = New Scripting.Dictionary
  Call GetWidthFor(data_.nodes_(1))
End Function

Public Function RepositionAll()
  Call SetPositionFor(data_.nodes_(1), dBaseX:=0)
End Function

' ==============
Private Function GetWidthFor(iNode As CDS_NodeSH) As Double
  If width_.Exists(iNode.id_) Then
    GetWidthFor = width_(iNode.id_)
  Else
    width_(iNode.id_) = CalculateWidthFor(iNode)
    GetWidthFor = width_(iNode.id_)
  End If
End Function

Private Function CalculateWidthFor(iNode As CDS_NodeSH) As Double
  Dim dChildWidth As Double
  Dim iChild As CDS_NodeSH
  Dim dNodeWidth As Double: dNodeWidth = iNode.data_.shape_.CellsU("Width")
  If iNode.children_.Count = 0 Then
    CalculateWidthFor = dNodeWidth
  ElseIf GetLayoutFor(iNode) = T_LAYOUT_VERTICAL Then
    dNodeWidth = dNodeWidth
    For Each iChild In iNode.children_
      dChildWidth = GetWidthFor(iChild)
      If dNodeWidth < dChildWidth Then _
        dNodeWidth = dChildWidth
    Next iChild
    CalculateWidthFor = dNodeWidth + GAP_VERT_ADJUSTMENT_TO_HOR
  Else
    Dim dAccumulated As Double: dAccumulated = -GAP_HORIZONTAL
    For Each iChild In iNode.children_
      dAccumulated = dAccumulated + GAP_HORIZONTAL + GetWidthFor(iChild)
    Next iChild

    Dim dRankWidth As Double: dRankWidth = GetRankWidth(iNode.rank_)
    If dAccumulated < dRankWidth Then _
      dAccumulated = dRankWidth
    If dAccumulated < dNodeWidth Then _
      dAccumulated = dNodeWidth
    CalculateWidthFor = dAccumulated
  End If
End Function

Private Function SetPositionFor(iNode As CDS_NodeSH, dBaseX As Double)
  Dim iTarget As Visio.Shape: Set iTarget = iNode.data_.shape_
  Dim dNodeWidth As Double: dNodeWidth = width_(iNode.id_)
  Dim iChild As CDS_NodeSH
  
  Call CSE_ProgressBar.IncrementA
  If iNode.children_.Count = 0 Then
    iTarget.CellsU("PinX") = dBaseX + dNodeWidth / 2#
  ElseIf GetLayoutFor(iNode) = T_LAYOUT_VERTICAL Then
    iTarget.CellsU("PinX") = dBaseX + iTarget.CellsU("Width") / 2# + GAP_VERT_ADJUSTMENT_TO_HOR
    For Each iChild In iNode.children_
      iChild.data_.shape_.CellsU("PinX") = dBaseX + iChild.data_.shape_.CellsU("Width") / 2# + GAP_VERT_ADJUSTMENT_TO_HOR
    Next iChild
    Call CSE_ProgressBar.IncrementA(iNode.children_.Count)
  Else ' Horizontal
    Dim newX As Double: newX = dBaseX
    If iNode.children_.Count = 1 And dNodeWidth > width_(iNode.children_(1).id_) Then _
      newX = dBaseX + dNodeWidth / 2# - width_(iNode.children_(1).id_) / 2#
    For Each iChild In iNode.children_
      Call SetPositionFor(iChild, newX)
      newX = newX + width_(iChild.id_) + GAP_HORIZONTAL
    Next iChild
    iTarget.CellsU("PinX") = iNode.children_(1).data_.shape_.CellsU("PinX") / 2# _
                            + iNode.children_(iNode.children_.Count).data_.shape_.CellsU("PinX") / 2#
  End If
End Function

Private Function MoveShapeAfterVertical(sh1 As Visio.Shape, sh2 As Visio.Shape, dGap As Double)
  Dim dPinx As Double: dPinx = sh2.CellsU("PinX") - sh1.CellsU("Width") / 2# + sh2.CellsU("Width") / 2#
  Dim dPiny As Double: dPiny = sh2.CellsU("PinY") - dGap - sh2.CellsU("Height") / 2# - sh1.CellsU("Height") / 2#
  Call sh1.SetCenter(dPinx, dPiny)
End Function

Private Function MoveShapeAfterHorizontal(sh1 As Visio.Shape, sh2 As Visio.Shape)
  Dim dPinx As Double: dPinx = sh2.CellsU("PinX") + GAP_HORIZONTAL + sh2.CellsU("Width") / 2# + sh1.CellsU("Width") / 2#
  Dim dPiny As Double: dPiny = sh2.CellsU("PinY") + sh2.CellsU("Height") / 2# - sh1.CellsU("Height") / 2#
  Call sh1.SetCenter(dPinx, dPiny)
End Function

Private Function GetLayoutFor(iNode As CDS_NodeSH) As TLayout
  If iNode Is Nothing Then
    GetLayoutFor = T_LAYOUT_HORIZONTAL
  ElseIf iNode.parent_ Is Nothing Then
    GetLayoutFor = T_LAYOUT_HORIZONTAL
  ElseIf iNode.children_.Count <= 1 Then
    GetLayoutFor = T_LAYOUT_HORIZONTAL
  ElseIf Not autoVertical_ And iNode.rank_ + 1 <> data_.MaxDepth Then
    GetLayoutFor = T_LAYOUT_HORIZONTAL
  ElseIf iNode.descendantsCount_ = iNode.children_.Count Then
    GetLayoutFor = T_LAYOUT_VERTICAL
  Else
    GetLayoutFor = T_LAYOUT_HORIZONTAL
  End If
End Function

Private Function CreateNodeShape(ByRef iNode As CDS_NodeSH) As Visio.Shape
  Dim iShape As Visio.Shape: Set iShape = page_.Drop(GetRankMaster(iNode.rank_), 0, 0)
  With iShape
    .Text = iNode.data_.text_
    .Name = PREFIX_NODE & iNode.id_
    .CellsSRC(visSectionObject, visRowLock, visLockDelete).FormulaU = "0"
    If iNode.data_.color_ <> COLOR_DEFAULT Then _
      .CellsSRC(visSectionObject, visRowFill, visFillForegnd).FormulaU = ConvertRGBtoString(iNode.data_.color_)
  End With
  Set CreateNodeShape = iShape
End Function

Private Function TryOptimizeHeight(iShape As Visio.Shape, nRank&, nLayout As TLayout)
  Dim dGap As Double: dGap = IIf(nLayout = T_LAYOUT_HORIZONTAL, GAP_RANK, GAP_VERTICAL)
  Dim iPrototype As Visio.Master: Set iPrototype = GetRankMaster(nRank)
  Dim kFactor As Double: kFactor = VBA.Sqr(iShape.CellsU("Width") / iPrototype.Shapes(1).CellsU("Width"))
  If kFactor < SIZING_FACTOR_LIMIT Then _
    Exit Function
  
  Dim dLeft As Double: dLeft = iShape.CellsU("PinX") - iShape.CellsU("Width") / 2#
  Dim dTop As Double: dTop = iShape.CellsU("PinY") + iShape.CellsU("Height") / 2#
  kFactor = VBA.Int(kFactor) + 1
  iShape.CellsU("Height") = iPrototype.Shapes(1).CellsU("Height") * kFactor + dGap * (kFactor - 1)
  iShape.CellsU("PinY") = dTop - iShape.CellsU("Height") / 2#
  If nLayout = T_LAYOUT_HORIZONTAL Then _
    iShape.CellsU("PinX") = dLeft + iShape.CellsU("Width") / 2#
End Function
