Attribute VB_Name = "Main"
Option Explicit

Public Const PRODUCT_VERSION = "1.2.0"
Public Const PRODUCT_NAME = "Concept-Hierarchy"

Public g_VersionTimer As Long

Public Sub StartVersionCheck()
  g_VersionTimer = SetTimer(0, 0, CP_VERSION_MSG_DELAY, AddressOf OnVersionCheck)
End Sub

Public Function OnVersionCheck(ByVal nHwnd As Long, ByVal uMsg As Long, ByVal nEvent As Long, ByVal nTime As Long)
  Call KillTimer(0, g_VersionTimer)
  Call VersionValidate(PRODUCT_NAME, PRODUCT_VERSION)
End Function

Public Sub RunGotoLinkBegin()
Attribute RunGotoLinkBegin.VB_ProcData.VB_Invoke_Func = "Q"
  Call CC_GotoLinkBegin
End Sub

Public Sub RunGotoLinkEnd()
  Call CC_GotoLinkEnd
End Sub

Public Sub RunImportWord()
  Dim sFile$: sFile = UserInteraction.PromptFileFilter( _
                        sInitialPath:=Visio.ActiveDocument.Path, _
                        sDescription:=" ,       ", _
                        sFilter:="*.docx;*.doc;*.docm", _
                        sTitle:=" Word", _
                        bNewApplication:=True)
  If sFile = vbNullString Then _
    Exit Sub
  Dim iData As CDS_StaticHierarchy: Set iData = ScanHierarchy(sFile, MODE_WORD)
  If iData Is Nothing Then _
    Exit Sub
  
  Call OutputHierarchy(iData, MODE_VISIO)
End Sub

Public Sub RunImportXL()
  Dim sFile$: sFile = UserInteraction.PromptFileFilter( _
                        sInitialPath:=Visio.ActiveDocument.Path, _
                        sDescription:=" ,       ", _
                        sFilter:="*.xlsx;*.xls;*.xlsm", _
                        sTitle:=" Excel", _
                        bNewApplication:=True)
  If sFile = vbNullString Then _
    Exit Sub
  Dim iData As CDS_StaticHierarchy: Set iData = ScanHierarchy(sFile, MODE_EXCEL)
  If iData Is Nothing Then _
    Exit Sub
  
  Call OutputHierarchy(iData, MODE_VISIO)
End Sub

Public Sub RunMaster()
  Call MasterDlg.Show
  If MasterDlg.bCanceled Then _
    Exit Sub
  
  Dim iMode As IOMode: iMode = MasterDlg.InputMode
  Dim oMode As IOMode: oMode = MasterDlg.OutputMode
  Dim iLayout As TLayout: iLayout = MasterDlg.InputLayout
  Dim sFileName$: sFileName = MasterDlg.InputFile
  
  If iMode <> MODE_VISIO And sFileName = vbNullString Then
    Call UserInteraction.ShowMessage(EM_FILENAME_EMPTY)
    Exit Sub
  End If
  
  Dim iData As CDS_StaticHierarchy: Set iData = ScanHierarchy(sFileName, iMode, iLayout)
  If iData Is Nothing Then _
    Exit Sub
  
  Call OutputHierarchy(iData, MasterDlg.OutputMode)
End Sub

Public Sub RunClearScheme()
Attribute RunClearScheme.VB_Description = " "
Attribute RunClearScheme.VB_ProcData.VB_Invoke_Func = "X"
  Call GlobalUndo.BeginScope("  ")
  
  Call ClearGeneratedShapes(MainPage)
  Call VsoShowWholePage(MainPage)
  
  Call GlobalUndo.EndScope
End Sub

Public Sub RunRename()
Attribute RunRename.VB_Description = ""
  Dim sel As Visio.Selection:  Set sel = Visio.Application.ActiveWindow.Selection
  If sel.Count <> 1 Then
    Call UserInteraction.ShowMessage(EM_INVALID_SELECTION)
    Exit Sub
  End If
  
  Call GlobalUndo.BeginScope("  ")
  
  Call GenerateNamesFor(sel.Item(1))
  
  Call GlobalUndo.EndScope
End Sub

Public Sub RunTransform()
Attribute RunTransform.VB_ProcData.VB_Invoke_Func = "Z"
  Dim sel As Visio.Selection: Set sel = Visio.Application.ActiveWindow.Selection
  If sel.Count <> 1 Then
    Call UserInteraction.ShowMessage(EM_INVALID_SELECTION)
    Exit Sub
  End If
  
  Call GlobalUndo.BeginScope(" ")
  
  Call TransformNodeLayout(sel.Item(1))
  
  Call GlobalUndo.EndScope
End Sub

Public Sub RunConnectColumns()
  Dim sel As Visio.Selection: Set sel = Visio.Application.ActiveWindow.Selection
  If sel.Count <> 1 Then
    Call UserInteraction.ShowMessage(EM_INVALID_SELECTION)
    Exit Sub
  End If
  
  Call GlobalUndo.BeginScope("  ")
  
  Call ConnectColumns(sel.Item(1))
  
  Call GlobalUndo.EndScope
End Sub

Public Sub RunNewRankTemplate()
  Dim iMaster As Visio.Master
  Set iMaster = AddNewRankTemplate
  If Not iMaster Is Nothing Then _
    Call UserInteraction.ShowMessage(IM_NEW_TEMPLATE, iMaster.Name)
End Sub

' =======
Private Function ScanHierarchy(sFileName$, iMode As IOMode, Optional nLayout As TLayout = T_LAYOUT_HORIZONTAL) As CDS_StaticHierarchy
  Dim iData As CDS_StaticHierarchy
  Select Case iMode
    Case MODE_EXCEL: Set iData = ProcessExcel(sFileName)
    Case MODE_WORD: Set iData = ProcessWord(sFileName)
    Case MODE_VISIO: Set iData = ProcessVisio(VsoGetSelectedShapes(ThisDocument.Application), nLayout)
  End Select
  If iData Is Nothing Then _
    Exit Function
  If Not ValidateHierarchy(iData) Then
    Call UserInteraction.ShowMessage(EM_INVALID_STRUCTURE)
    Exit Function
  End If
  Set ScanHierarchy = iData
End Function

Private Function OutputHierarchy(iData As CDS_StaticHierarchy, oMode As IOMode)
  Select Case oMode
    Case MODE_EXCEL: Call GenerateExcel(iData, Not MasterDlg.XLExportAsTree)
    Case MODE_WORD: Call GenerateWord(iData, MasterDlg.WordExportAsString)
    Case MODE_VISIO: Call GenerateVisio(MainPage, iData, MasterDlg.AutoLayout)
  End Select
End Function

Private Function ProcessVisio(iShapes As Collection, nLayout As TLayout) As CDS_StaticHierarchy
  If iShapes.Count <> 1 Then
    Call UserInteraction.ShowMessage(EM_INVALID_SELECTION)
    Exit Function
  End If
  Set ProcessVisio = ScanVisioNode(iShapes(1), nLayout)
End Function

Private Function ProcessExcel(sFileName$) As CDS_StaticHierarchy
  Dim xlApp As New API_XLWrapper
  If xlApp.OpenDocument(sFileName, bReadOnly:=True) Is Nothing Then
    Call UserInteraction.ShowMessage(EM_ERROR_LOADING_FILE)
    Exit Function
  End If
  
  Dim iSource As Excel.Worksheet: Set iSource = xlApp.Document.Worksheets(1)
  Call CSE_ProgressBar.Init(" ", maxVal:=iSource.UsedRange.Rows.Count, curVal:=1)
  Call CSE_ProgressBar.ShowModeless
  
  Set ProcessExcel = ScanExcel(iSource)
  If ProcessExcel Is Nothing Then _
    Call UserInteraction.ShowMessage(EM_INVALID_FORMAT, xlApp.Document.Name)
  
  Call Unload(CSE_ProgressBar)
  Call xlApp.ReleaseDocument
End Function

Private Function ProcessWord(sFileName$) As CDS_StaticHierarchy
  Dim wordApp As New API_WordWrapper
  If wordApp.OpenDocument(sFileName, bReadOnly:=True) Is Nothing Then
    Call UserInteraction.ShowMessage(EM_ERROR_LOADING_FILE)
    Exit Function
  End If
  
  Call CSE_ProgressBar.Init("  ", maxVal:=wordApp.Document.Paragraphs.Count, curVal:=0)
  Call CSE_ProgressBar.ShowModeless
  
  Set ProcessWord = ScanWord(wordApp.Document)
  If ProcessWord Is Nothing Then _
    Call UserInteraction.ShowMessage(EM_INVALID_FORMAT, wordApp.Document.Name)
  
  Call Unload(CSE_ProgressBar)
  Call wordApp.ReleaseDocument
End Function

Private Function GenerateWord(iData As CDS_StaticHierarchy, bAsString As Boolean)
  If Not bAsString And iData.MaxDepth > WORD_HEADING_MAX_LEVEL Then
    Call UserInteraction.ShowMessage(EM_WORD_MAX_LEVELS)
    Exit Function
  End If

  Dim sTemplate$: sTemplate = VBA.Environ("AppData") & EXPORT_TEMPLATE_PATH
  Dim wordApp As New API_WordWrapper
  Dim iOutput As Word.Document: Set iOutput = wordApp.NewDocument(sTemplate)
  Call iOutput.Range.Delete
  
  Call wordApp.PauseUI
  If Not bAsString Then
    Call Hierarchy2Word(iData, iOutput)
  Else
    Call CSE_ProgressBar.Init(" ...", maxVal:=iData.Size, curVal:=1)
    Call CSE_ProgressBar.ShowModeless
    
    Call Hierarchy2WordString(iData, iOutput)
    
    Call Unload(CSE_ProgressBar)
  End If
  Call UserInteraction.ShowMessage(IM_EXPORT_OK, iData.Size, iData.MaxDepth)
  Call wordApp.ResumeUI
End Function

Private Function GenerateExcel(iData As CDS_StaticHierarchy, bFlatData As Boolean)
  Dim xlApp As New API_XLWrapper
  Dim xlOut As Excel.Workbook: Set xlOut = xlApp.NewDocument
  
  Call xlApp.PauseUI
  Call Hierarchy2XL(iData, xlOut.Worksheets(1), bFlatData)
  Call UserInteraction.ShowMessage(IM_EXPORT_OK, iData.Size, iData.MaxDepth)
  Call xlApp.ResumeUI
End Function

Private Function GenerateVisio(iPage As Visio.Page, iData As CDS_StaticHierarchy, Optional bAutoVertical As Boolean = True)
  Call ClearGeneratedShapes(iPage)
  Dim iLayout As New LayoutConstruction: Call iLayout.Init(iPage, iData, bAutoVertical)
  
  Dim vsoApp As New API_VsoWrapper:  Call vsoApp.SetDocument(ThisDocument)
  Call vsoApp.PauseUI
  Call GlobalUndo.BeginScope("  ")
  
  Call CSE_ProgressBar.Init("   ", maxVal:=iData.Size, curVal:=1)
  Call CSE_ProgressBar.ShowModeless
  
  Call iLayout.ConstructHNodes
  DoEvents
  
  Call CSE_ProgressBar.Init(" ", maxVal:=iData.Size)
  Call iLayout.CalculateWidth
  Call iLayout.RepositionAll
  
  Call Unload(CSE_ProgressBar)
  
  Call GlobalUndo.EndScope
  Call vsoApp.ResumeUI
      
  Call iPage.ResizeToFitContents
  Call VsoShowWholePage(iPage)
  
  Dim nWidth&: nWidth = iPage.PageSheet.CellsU("PageWidth").Result("mm")
  Dim nHeight&: nHeight = iPage.PageSheet.CellsU("PageHeight").Result("mm")
  Call UserInteraction.ShowMessage(IM_IMPORT_OK, iData.Size, iData.MaxDepth, nWidth, nHeight)
End Function
