Attribute VB_Name = "MainImpl"
Option Explicit

Public Function ClearGeneratedShapes(target As Visio.Page)
  Dim nItem&: nItem = 1
  Do While Not nItem > target.Shapes.Count
    Dim vsoShape As Shape: Set vsoShape = target.Shapes(nItem)
    Dim shpName$: shpName = vsoShape.Name
    If Left(shpName, 2) = "#N" Or Left(shpName, 2) = "#C" Then
      vsoShape.Delete
      nItem = 1
    Else
      nItem = nItem + 1
    End If
  Loop
  Call target.ResizeToFitContents
End Function

Public Function ValidateHierarchy(target As CDS_StaticHierarchy) As Boolean
  ValidateHierarchy = False
  If target.nodes_.Count = 0 Then _
    Exit Function
  If target.nodes_(1).rank_ <> 1 Then _
    Exit Function
  
  Dim iNode As CDS_NodeSH
  For Each iNode In target.nodes_
    If iNode.id_ <> 1 And iNode.rank_ <= 1 Then _
      Exit Function
  Next iNode
  
  ValidateHierarchy = True
End Function

Public Function ConnectNodes(shape1 As Visio.Shape, shape2 As Visio.Shape, nLayout As TLayout) As Boolean
  Dim iConnector As Visio.Shape: Set iConnector = CreateConnector(shape1, shape2, nLayout)
  iConnector.CellsSRC(visSectionObject, visRowLock, visLockDelete).FormulaU = "0"
  iConnector.Name = PREFIX_CONNECTOR & shape1.ID & "-" & shape2.ID
End Function

Public Function CreateConnector(sh1 As Visio.Shape, sh2 As Visio.Shape, nMode As TLayout) As Visio.Shape
  Dim iPage As Visio.Page:  Set iPage = sh1.ContainingPage
  Dim prototype As Visio.Master: Set prototype = GetConnectionMaster(nMode)
  
  Call sh1.AutoConnect(sh2, visAutoConnectDirNone, prototype)
  Dim connector As Visio.Shape: Set connector = iPage.Shapes(iPage.Shapes.Count)
  Select Case nMode
    Case T_LAYOUT_HORIZONTAL
      Call connector.CellsU("BeginX").GlueTo(sh1.CellsSRC(7, 0, 0))
      Call connector.CellsU("EndX").GlueTo(sh2.CellsSRC(7, 1, 0))
    
    Case T_LAYOUT_COLUMN_LEFT
      Call connector.CellsU("BeginX").GlueTo(sh1.CellsSRC(7, 0, 0))
      Call connector.CellsU("EndX").GlueTo(sh2.CellsSRC(7, 3, 0))
      
    Case T_LAYOUT_COLUMN_RIGHT
      Call connector.CellsU("BeginX").GlueTo(sh1.CellsSRC(7, 0, 0))
      Call connector.CellsU("EndX").GlueTo(sh2.CellsSRC(7, 2, 0))
    
    Case Else
      Call connector.CellsU("BeginX").GlueTo(sh1.CellsSRC(7, 2, 0))
      Call connector.CellsU("EndX").GlueTo(sh2.CellsSRC(7, 2, 0))
  End Select
  With connector
    .CellsSRC(visSectionObject, visRowShapeLayout, visSLORouteStyle).FormulaU = 8
    .CellsSRC(visSectionObject, visRowShapeLayout, visSLORouteStyle).FormulaU = _
      prototype.Shapes(1).CellsSRC(visSectionObject, visRowShapeLayout, visSLORouteStyle).FormulaU
  End With
  Set CreateConnector = connector
End Function

Public Function ScanExcel(iSource As Excel.Worksheet) As CDS_StaticHierarchy
  Dim iData As New CDS_StaticHierarchy
  If iSource.Cells(1, 1) = vbNullString Then _
    Exit Function
  
  Dim nRow&
  For nRow = 1 To iSource.UsedRange.Rows.Count Step 1
    Dim iNode As CDS_NodeSH: Set iNode = iData.PushItem(iSource.Cells(nRow, 1))
    If iNode Is Nothing Then _
      Exit Function
    Set iNode.data_ = ExrtactXLPayload(iSource, nRow)
    Call CSE_ProgressBar.IncrementA
  Next nRow
  
  Set ScanExcel = iData
End Function

Public Function ScanWord(iSource As Word.Document) As CDS_StaticHierarchy
' Scan word file to create DB
  Dim iData As New CDS_StaticHierarchy
  Dim iRoot As CDS_NodeSH: Set iRoot = iData.PushItem(1)
  Set iRoot.data_ = New VisioNode
  iRoot.data_.text_ = iSource.Name
  
  Dim aPar As Word.Paragraph
  Dim pars As Word.Paragraphs: Set pars = iSource.Paragraphs
  Dim nColor&
  For Each aPar In pars
    Dim nRank&: nRank = aPar.OutlineLevel
    If nRank = wdOutlineLevelBodyText Then _
      GoTo NEXT_PAR
    Dim iPayload As VisioNode: Set iPayload = ExtractWordPayload(aPar.Range)
    If iPayload Is Nothing Then _
      GoTo NEXT_PAR

    Dim iNode As CDS_NodeSH: Set iNode = iData.PushItem(nRank + 1)
    If iNode Is Nothing Then _
      Exit Function
    Set iNode.data_ = iPayload
    
NEXT_PAR:
    Call CSE_ProgressBar.IncrementA
  Next aPar
  
  Set ScanWord = iData
End Function

Public Function ScanVisioNode(iRoot As Visio.Shape, iLayout As TLayout) As CDS_StaticHierarchy
  Dim iData As New CDS_StaticHierarchy
  Call ScanShapeRecur(iRoot, iData, iLayout)
  Set ScanVisioNode = iData
End Function

Public Function Hierarchy2XL(iData As CDS_StaticHierarchy, ByRef iOut As Excel.Worksheet, bFlatData As Boolean)
  Dim iNode As CDS_NodeSH
  Dim nRow&: nRow = 1
  For Each iNode In iData.nodes_
    If bFlatData Then
      iOut.Cells(nRow, 1) = iNode.rank_
      iOut.Cells(nRow, 2) = iNode.data_.text_
      If iNode.data_.color_ <> COLOR_DEFAULT Then _
        iOut.Cells(nRow, 2).Interior.Color = iNode.data_.color_
    Else
      iOut.Cells(nRow, iNode.rank_) = iNode.data_.text_
      If iNode.data_.color_ <> COLOR_DEFAULT Then _
        iOut.Cells(nRow, iNode.rank_).Interior.Color = iNode.data_.color_
    End If
    nRow = nRow + 1
  Next iNode
End Function

Public Function Hierarchy2Word(iData As CDS_StaticHierarchy, ByRef target As Word.Document)
  If target Is Nothing Then _
    Exit Function
  
  Dim iNode As CDS_NodeSH
  Dim iInsert As Word.Range: Set iInsert = target.Range
  For Each iNode In iData.nodes_
    Dim nColor&: nColor = iNode.data_.color_
    Set iInsert = WordAddLine(iNode.data_.text_, iInsert, WordStyleForRank(target, iNode.rank_))
    If nColor = COLOR_DEFAULT Then
      iInsert.Font.Shading.BackgroundPatternColor = wdColorAutomatic
    Else
      iInsert.Font.Shading.BackgroundPatternColor = nColor
    End If
  Next iNode
End Function

Public Function Hierarchy2WordString(iData As CDS_StaticHierarchy, ByRef target As Word.Document)
  Const SYMBOL_MASK_TOTAL = "[@#]"
  Const SYMBOL_MASK_BREAK = "@"

  Dim iStack As New Collection
  Call iStack.Add(iData.nodes_(1))
  
  Dim childrenIDs As New Collection
  Dim curChild&
  Do While iStack.Count > 0
    Dim iNode As CDS_NodeSH: Set iNode = iStack.Item(iStack.Count)
    Dim sLastChar$: sLastChar = IIf(VBA.Len(iNode.data_.text_) > 0, VBA.Right(iNode.data_.text_, 1), "")
    If sLastChar Like SYMBOL_MASK_TOTAL Or iNode.children_.Count = 0 Then
    ' Terminal
      Call WordAddLine(StackToString(iStack), target.Range, "")
      Call iStack.Remove(iStack.Count)
      Call CSE_ProgressBar.IncrementA
      If sLastChar Like SYMBOL_MASK_BREAK Then _
        Exit Do
    ElseIf childrenIDs.Count < iStack.Count Then
    ' Moving down
      Call iStack.Add(iNode.children_(1))
      Call childrenIDs.Add(1)
    Else
      curChild = childrenIDs.Item(childrenIDs.Count)
      If curChild < iNode.children_.Count Then
      ' Move to next child
        Call childrenIDs.Remove(childrenIDs.Count)
        Call iStack.Add(iNode.children_(curChild + 1))
        Call childrenIDs.Add(curChild + 1)
      Else
      ' Moving up
        Call childrenIDs.Remove(childrenIDs.Count)
        Call iStack.Remove(iStack.Count)
      End If
    End If
  Loop
End Function

Public Function GenerateNamesFor(tRoot As Visio.Shape)
  Dim oldName$: oldName = tRoot.Text
  If oldName Like "#* *" Then
    Call NameTheNode(VBA.Left(oldName, VBA.InStr(1, oldName, " ") - 1), tRoot)
  Else
    Call NameTheNode("", tRoot)
  End If
End Function

Public Function TransformNodeLayout(rootShape As Visio.Shape)
  Dim iData As CDS_StaticHierarchy: Set iData = ScanVisioNode(rootShape, T_LAYOUT_HORIZONTAL)
  If iData Is Nothing Then
    Call UserInteraction.ShowMessage(EM_INVALID_STRUCTURE)
    Exit Function
  End If
  
  If Not iData.MaxDepth >= 2 Then
    Call UserInteraction.ShowMessage(EM_CANNOT_TRANSFORM_DEEP)
    Exit Function
  End If
  
  Dim rootNode As CDS_NodeSH: Set rootNode = iData.nodes_(1)
  Dim nChildrenCount&: nChildrenCount = rootNode.children_.Count
  If nChildrenCount < 2 Then _
    Exit Function
  
  Dim oChildren As New Collection
  Dim aNode As CDS_NodeSH
  For Each aNode In rootNode.children_
    Call oChildren.Add(aNode.data_.shape_)
  Next aNode
  
  Dim theLayout As TLayout: theLayout = T_LAYOUT_HORIZONTAL
  If oChildren(1).CellsU("PinY") - oChildren(2).CellsU("PinY") < FLOAT_ACCURACY Then _
    theLayout = T_LAYOUT_VERTICAL
  
  Call SetLayout(rootShape, oChildren, theLayout)
End Function

Public Function AddNewRankTemplate() As Visio.Master
  Dim nRank&: nRank = 1
  Do While Not FindMaster(ThisDocument, MASTER_RANK & nRank) Is Nothing
    nRank = nRank + 1
  Loop
  Dim iMaster As Visio.Master: Set iMaster = ThisDocument.Masters.Add
  iMaster.Name = MASTER_RANK & nRank
  Dim iShape As Visio.Shape: Set iShape = iMaster.Drop(GetRankMaster(nRank - 1), 0, 0)
  iShape.Name = SHAPE_RANK_MASTER & nRank
  iShape.Text = " " & nRank
  Set AddNewRankTemplate = iMaster
End Function

Public Function ConnectColumns(rootShape As Visio.Shape)
  Dim iData As CDS_StaticHierarchy: Set iData = ScanVisioNode(rootShape, T_LAYOUT_VERTICAL)
  If iData Is Nothing Then
    Call UserInteraction.ShowMessage(EM_INVALID_STRUCTURE)
    Exit Function
  End If
  
  Dim dMidX As Double: dMidX = rootShape.CellsU("PinX")
  
  Dim iChild As Visio.Shape
  Dim iConnect As Visio.Connect
  Dim aNode As CDS_NodeSH
  For Each aNode In iData.nodes_(1).children_
    Set iChild = aNode.data_.shape_
    For Each iConnect In iChild.FromConnects
      If iConnect.FromSheet.Connects(1).ToSheet Is rootShape Then _
        Call iConnect.FromSheet.Delete
    Next iConnect
    
    If iChild.CellsU("PinX") > dMidX Then
      Call ConnectNodes(rootShape, iChild, T_LAYOUT_COLUMN_RIGHT)
    Else
      Call ConnectNodes(rootShape, iChild, T_LAYOUT_COLUMN_LEFT)
    End If
  Next aNode
End Function

' ========
Private Function NameTheNode(sID$, target As Visio.Shape)
  Dim oldName$: oldName = target.Text
  If VBA.IsNumeric(VBA.Left(oldName, 1)) Then _
    oldName = VBA.Right(oldName, VBA.Len(oldName) - VBA.InStr(1, oldName, " "))
  If sID <> vbNullString Then _
    target.Text = sID & " " & oldName
  
  Dim oChildren As New Collection: Set oChildren = GetChildrenOf(target)
  If oChildren.Count = 0 Then _
    Exit Function
  
  Dim nIndex() As Long: nIndex = SortChildrenCount(oChildren)

  Dim nItem&: nItem = 1
  Dim iChild As Visio.Shape
  For Each iChild In oChildren
    If sID = vbNullString Then
      Call NameTheNode(nIndex(nItem) & ".", iChild)
    Else
      Call NameTheNode(sID & nIndex(nItem) & ".", iChild)
    End If
    nItem = nItem + 1
  Next iChild
End Function

Private Function SortChildrenCount(oChildren As Collection) As Long()
  ' using CountSort algorithm
  Dim nIndicies() As Long
  ReDim nIndicies(1 To oChildren.Count)
  
  Dim sibling1 As Visio.Shape
  Dim nItem1&: nItem1 = 1
  For Each sibling1 In oChildren
    nIndicies(nItem1) = 1
    Dim dLeftX As Double: dLeftX = sibling1.CellsU("PinX") - sibling1.CellsU("Width") / 2#
    Dim dTopY As Double: dTopY = sibling1.CellsU("PinY") + sibling1.CellsU("Height") / 2#
    
    Dim nItem2&: nItem2 = 1
    Dim sibling2 As Visio.Shape
    For Each sibling2 In oChildren
      If nItem1 = nItem2 Then _
        GoTo NEXT_2
      
      Dim dLeftX2 As Double: dLeftX2 = sibling2.CellsU("PinX") - sibling2.CellsU("Width") / 2#
      Dim dTopY2 As Double: dTopY2 = sibling2.CellsU("PinY") + sibling2.CellsU("Height") / 2#
        
      If dTopY + FLOAT_ACCURACY < dTopY2 Then
        nIndicies(nItem1) = nIndicies(nItem1) + 1
      ElseIf Abs(dTopY - dTopY2) < FLOAT_ACCURACY And dLeftX - FLOAT_ACCURACY > dLeftX2 Then
        nIndicies(nItem1) = nIndicies(nItem1) + 1
      End If
NEXT_2:
      nItem2 = nItem2 + 1
    Next sibling2
    nItem1 = nItem1 + 1
  Next sibling1
  SortChildrenCount = nIndicies
End Function

Public Function SetLayout(theRoot As Visio.Shape, oChildren As Collection, theLayout As TLayout)
  Dim baseXPos As Double: baseXPos = theRoot.CellsU("PinX")
  Dim baseYPos As Double: baseYPos = theRoot.CellsU("PinY") - theRoot.CellsU("Height") / 2#
  
  Dim iChild As Visio.Shape
  If theLayout = T_LAYOUT_HORIZONTAL Then
    For Each iChild In oChildren
      baseXPos = baseXPos - GAP_HORIZONTAL / 2# - iChild.CellsU("Width") / 2#
    Next iChild
  End If

  Dim cnt As Visio.Connect
  For Each iChild In oChildren
    Dim dWidth As Double: dWidth = iChild.CellsU("Width")
    Dim dHeight As Double: dHeight = iChild.CellsU("Height")
    For Each cnt In iChild.FromConnects
      If cnt.FromSheet.Connects(1).ToSheet Is theRoot Then _
        Call cnt.FromSheet.Delete
    Next cnt
    
    iChild.CellsU("PinY") = baseYPos - GAP_VERTICAL - dHeight / 2#
    If theLayout = T_LAYOUT_VERTICAL Then
      iChild.CellsU("PinX") = baseXPos - theRoot.CellsU("Width") / 2# + dWidth / 2#
      baseYPos = baseYPos - GAP_VERTICAL - dHeight
    Else
      iChild.CellsU("PinX") = baseXPos + GAP_HORIZONTAL + dWidth / 2#
      baseXPos = baseXPos + GAP_HORIZONTAL + dWidth
    End If
    
    Call ConnectNodes(theRoot, iChild, theLayout)
  Next iChild
End Function

Private Function ScanShapeRecur(target As Visio.Shape, ByRef iData As CDS_StaticHierarchy, _
                                aLayout As TLayout, Optional iParent As CDS_NodeSH = Nothing)
  If target Is Nothing Then _
    Exit Function
  Dim nRank&: nRank = 1
  If Not iParent Is Nothing Then _
    nRank = iParent.rank_ + 1
  
  Dim iNode As CDS_NodeSH: Set iNode = iData.PushItem(nRank)
  Set iNode.data_ = ExtractVsoPayload(target)
  
  Dim oChildren As Collection: Set oChildren = GetChildrenOf(target)
  If oChildren.Count = 0 Then _
    Exit Function
  
  Dim nIndicies() As Long: nIndicies = SortChildrenLayout(oChildren, target, aLayout)
  Dim n&
  For n = 1 To oChildren.Count
    Call ScanShapeRecur(oChildren(nIndicies(n)), iData, aLayout, iNode)
  Next n
End Function

Private Function SortChildrenLayout(oChildren As Collection, aParent As Visio.Shape, iLayout As TLayout) As Long()
  ' using bubble sort algorithm
  Dim nCount&: nCount = oChildren.Count
  Dim nIndicies() As Long
  ReDim nIndicies(1 To nCount)
  Dim n&
  For n = 1 To nCount Step 1
    nIndicies(n) = n
  Next n
  If nCount = 1 Then
    SortChildrenLayout = nIndicies
    Exit Function
  End If
  
  Dim k&
  For n = 1 To nCount - 1 Step 1
    For k = n + 1 To nCount
      If CompareChildren(aParent, oChildren(nIndicies(n)), oChildren(nIndicies(k)), iLayout) Then
        nIndicies(n) = nIndicies(n) + nIndicies(k)
        nIndicies(k) = nIndicies(n) - nIndicies(k)
        nIndicies(n) = nIndicies(n) - nIndicies(k)
      End If
    Next k
  Next n
  SortChildrenLayout = nIndicies
End Function

Private Function CompareChildren(aParent As Visio.Shape, child1 As Visio.Shape, child2 As Visio.Shape, iLayout As TLayout) As Boolean
  Select Case iLayout
  Case T_LAYOUT_RADIAL
    CompareChildren = _
      aParent.DistanceFrom(child1, visSpatialIncludeDataGraphics) _
      - aParent.DistanceFrom(child2, visSpatialIncludeDataGraphics) _
      > FLOAT_ACCURACY
  Case T_LAYOUT_HORIZONTAL
    CompareChildren = child1.CellsU("PinX") - child2.CellsU("PinX") > FLOAT_ACCURACY
  Case T_LAYOUT_VERTICAL
    CompareChildren = child2.CellsU("PinY") - child1.CellsU("PinY") > FLOAT_ACCURACY
  End Select
End Function

Private Function StackToString(iStack As Collection) As String
  Dim theString$: theString = ""
  Dim bFirst As Boolean: bFirst = True
  Dim iNode As CDS_NodeSH
  For Each iNode In iStack
    If Not bFirst Then _
      theString = theString & " "
    bFirst = False
    theString = theString & iNode.data_.text_
  Next iNode
  StackToString = theString
End Function

